﻿local JDI = JDI_2D07EA85_B774_43bf_873C_67DFA1FFDCA1;

local MainFrame = JDI.GUI.MainFrame;
MainFrame:DoInit();

MainFrame:RegisterEvent("PLAYER_ENTERING_WORLD");
MainFrame:RegisterEvent("UNIT_INVENTORY_CHANGED");

local GetItemInfo, GetContainerNumSlots, GetContainerItemLink, GetInventoryItemLink, UnitIsEnemy, UnitInVehicle =
	  GetItemInfo, GetContainerNumSlots, GetContainerItemLink, GetInventoryItemLink, UnitIsEnemy, UnitInVehicle;

function JDI:PLAYER_ENTERING_WORLD()
	if (not JoustDoItDB) then
		JoustDoItDB = {};
	end
	
	JDI.CharName = (UnitName("player")) .. " - " .. (GetRealmName());
	if (JoustDoItDB[JDI.CharName]) then
		JDI.Vars = JoustDoItDB[JDI.CharName];
	else
		JoustDoItDB[JDI.CharName] = {};
		JDI.Vars = JoustDoItDB[JDI.CharName];
	end
	
	-- Defaults
	if (JDI.Vars.Enable == nil) then
		JDI.Vars.Enable = true;
	end
	if (JDI.Vars.Target == nil) then
		JDI.Vars.Target = true;
	end
	if (JDI.Vars.Dismount == nil) then
		JDI.Vars.Dismount = true;
	end
	
	if (JDI.Vars.Enable) then
		MainFrame:RegisterEvent("UI_ERROR_MESSAGE");
		MainFrame:RegisterEvent("UNIT_EXITED_VEHICLE");
		if (JDI.Vars.Target) then
			MainFrame:RegisterEvent("PLAYER_TARGET_CHANGED");
			MainFrame:RegisterEvent("UNIT_ENTERING_VEHICLE");
			MainFrame:RegisterEvent("UNIT_ENTERED_VEHICLE");
		end
	end
	
	JDI:UNIT_INVENTORY_CHANGED();
end

function JDI:UNIT_INVENTORY_CHANGED()
	local link = GetInventoryItemLink("player", JDI.MAINHANDSLOT);
	if (link) then
		if (JDI:IsLance(link)) then
			-- Do nothing.
		else
			JDI.Vars.MainHand = link;
			link = GetInventoryItemLink("player", JDI.OFFHANDSLOT);
			if (JDI.Vars.OffHand and link) then
				JDI.Vars.OffHand = nil;
			end
		end
	end
end

function JDI:UI_ERROR_MESSAGE(msg)
	if (msg == SPELL_FAILED_CUSTOM_ERROR_60) then
		local link, equipt = JDI:FindLance();
		if (link and equipt) then
			-- Do nothing.
		elseif (link) then
			if (JDI.Vars.Dismount and IsMounted()) then
				Dismount();
			end
			local olink = GetInventoryItemLink("player", JDI.OFFHANDSLOT);
			if (olink and (select(9, GetItemInfo(olink))) == "INVTYPE_2HWEAPON") then
				for i = 0, NUM_BAG_SLOTS+1 do
					if (i == NUM_BAG_SLOTS+1) then
						JDI:Warn(JDI.Locale.NoBagSpace, "Sound\\Spells\\PVPFlagTaken.wav");
						break;
					end
					if (GetContainerNumFreeSlots(i) > 0) then
						JDI.Vars.OffHand = olink;
						PickupInventoryItem(JDI.OFFHANDSLOT);
						if (i == 0) then
							PutItemInBackpack();
						else
							PutItemInBag(JDI.BAGSLOT[i]);
						end
						break;
					end
				end
			end
			EquipItemByName(link, JDI.MAINHANDSLOT);
		else
			JDI:Warn(JDI.Locale.NoLance, "Sound\\Spells\\PVPFlagTaken.wav");
		end
	end
end

function JDI:UNIT_ENTERING_VEHICLE(unit)
	if (unit ~= "player") then
		return;
	end
	JDI.Entering = GetTime() + 5;
end

function JDI:UNIT_ENTERED_VEHICLE(unit)
	if (unit ~= "player") then
		return;
	end
	JDI.Entering = false;
end

function JDI:UNIT_EXITED_VEHICLE(unit)
	if (unit ~= "player") then
		return;
	end
	local link = GetInventoryItemLink("player", JDI.MAINHANDSLOT);
	if ((link and JDI:IsLance(link)) or (link == nil)) then
		local wpnlink, reason = JDI:FindWeapon(JDI.Vars.MainHand, JDI.MAINHANDSLOT);
		if (wpnlink and reason) then
			-- Do nothing.
		elseif (wpnlink) then
			EquipItemByName(JDI.Vars.MainHand, JDI.MAINHANDSLOT);
		else
			if (reason == 1) then
				JDI:Warn(JDI.Locale.WeaponNotSet, "Sound\\Spells\\PVPFlagTaken.wav");
			elseif (reason == 2) then
				JDI:Warn(format(JDI.Locale.WeaponNotFound, (GetItemInfo(JDI.Vars.MainHand))), "Sound\\Spells\\PVPFlagTaken.wav");
			end
		end
		wpnlink, reason = JDI:FindWeapon(JDI.Vars.OffHand, JDI.OFFHANDSLOT);
		if (wpnlink and reason) then
			-- Do nothing.
		elseif (wpnlink) then
			EquipItemByName(JDI.Vars.OffHand, JDI.OFFHANDSLOT);
		else
			if (reason == 1) then
				-- Do nothing. (Didn't swap Off Hand)
			elseif (reason == 2) then
				JDI:Warn(format(JDI.Locale.WeaponNotFound, (GetItemInfo(JDI.Vars.OffHand))), "Sound\\Spells\\PVPFlagTaken.wav");
			end
		end
	end
end

function JDI:PLAYER_TARGET_CHANGED()
	if (JDI.Entering and (GetTime() < JDI.Entering)) then
		return;
	end
	JDI.Entering = false;
	local link = GetInventoryItemLink("player", JDI.MAINHANDSLOT);
	if (link and JDI:IsLance(link) and UnitIsEnemy("player","target") and not UnitInVehicle("player")) then
		local wpnlink, reason = JDI:FindWeapon(JDI.Vars.MainHand, JDI.MAINHANDSLOT);
		if (wpnlink and reason) then
			-- Do nothing.
		elseif (wpnlink) then
			EquipItemByName(JDI.Vars.MainHand, JDI.MAINHANDSLOT);
		else
			if (reason == 1) then
				JDI:Warn(JDI.Locale.WeaponNotSet, "Sound\\Spells\\PVPFlagTaken.wav");
			elseif (reason == 2) then
				JDI:Warn(format(JDI.Locale.WeaponNotFound, (GetItemInfo(JDI.Vars.MainHand))), "Sound\\Spells\\PVPFlagTaken.wav");
			end
		end
		wpnlink, reason = JDI:FindWeapon(JDI.Vars.OffHand, JDI.OFFHANDSLOT);
		if (wpnlink and reason) then
			-- Do nothing.
		elseif (wpnlink) then
			EquipItemByName(JDI.Vars.OffHand, JDI.OFFHANDSLOT);
		else
			if (reason == 1) then
				-- Do nothing. (Didn't swap Off Hand)
			elseif (reason == 2) then
				JDI:Warn(format(JDI.Locale.WeaponNotFound, (GetItemInfo(JDI.Vars.OffHand))), "Sound\\Spells\\PVPFlagTaken.wav");
			end
		end
	end
end

function JDI:SetEnable()
	JDI.Vars.Enable = not JDI.Vars.Enable;
	if (JDI.Vars.Enable) then
		MainFrame:RegisterEvent("UI_ERROR_MESSAGE");
		MainFrame:RegisterEvent("UNIT_EXITED_VEHICLE");
		if (JDI.Vars.Target) then
			MainFrame:RegisterEvent("PLAYER_TARGET_CHANGED");
			MainFrame:RegisterEvent("UNIT_ENTERING_VEHICLE");
			MainFrame:RegisterEvent("UNIT_ENTERED_VEHICLE");
		end
	else
		MainFrame:UnregisterEvent("UI_ERROR_MESSAGE");
		MainFrame:UnregisterEvent("UNIT_EXITED_VEHICLE");
		if (JDI.Vars.Target) then
			MainFrame:UnregisterEvent("PLAYER_TARGET_CHANGED");
			MainFrame:UnregisterEvent("UNIT_ENTERING_VEHICLE");
			MainFrame:UnregisterEvent("UNIT_ENTERED_VEHICLE");
		end
	end
end

function JDI:SetOptTarget()
	JDI.Vars.Target = not JDI.Vars.Target;
	if (JDI.Vars.Enable and JDI.Vars.Target) then
		MainFrame:RegisterEvent("PLAYER_TARGET_CHANGED");
		MainFrame:RegisterEvent("UNIT_ENTERING_VEHICLE");
		MainFrame:RegisterEvent("UNIT_ENTERED_VEHICLE");
	else
		MainFrame:UnregisterEvent("PLAYER_TARGET_CHANGED");
		MainFrame:UnregisterEvent("UNIT_ENTERING_VEHICLE");
		MainFrame:UnregisterEvent("UNIT_ENTERED_VEHICLE");
	end
end

function JDI:SetOptDismount()
	JDI.Vars.Dismount = not JDI.Vars.Dismount;
end

--[[
	JDI:FindLance
	returns (itemlink, true) if in mainhand already
	returns (itemlink, false) if in bags
	returns nil if not found
]]
function JDI:FindLance()
	local link = GetInventoryItemLink("player", JDI.MAINHANDSLOT);
	local itemid = tonumber((select(2,strsplit(":", link))));
	if (itemid == 46069 or itemid == 46070 or itemid == 46106) then
		return link, true;
	end	
	
	for i = 0, 4 do
		for j = 1, GetContainerNumSlots(i) do
			link = GetContainerItemLink(i, j);
			if (link) then
				itemid = tonumber((select(2,strsplit(":", link))));
				if (itemid == 46069 or itemid == 46070 or itemid == 46106) then
					return link, false;
				end
			end
		end
	end
	
	return nil;
end

--[[
	Same returns as JDI:FindLance
	except returns (false, reason) if not found
	reason: 1 MainHand not set
		  : 2 Not in bags
]]
function JDI:FindWeapon(weapon, slot)
	local findname;
	if (weapon) then
		findname = GetItemInfo(weapon);
	end
	if (not findname) then
		return false, 1;
	end
	
	local link = GetInventoryItemLink("player", slot);
	local name;
	if (link) then
		name = GetItemInfo(link);
	end
	
	if (name == findname) then
		return link, true;
	end
	
	for i = 0, 4 do
		for j = 1, GetContainerNumSlots(i) do
			link = GetContainerItemLink(i, j);
			if (link) then
				name = GetItemInfo(link);
				if (name == findname) then
					return link, false;
				end
			end
		end
	end
	
	return false, 2;
end

function JDI:IsLance(itemlink)
	local itemid = tonumber((select(2,strsplit(":", itemlink))));
	if (itemid == 46069 or itemid == 46070 or itemid == 46106) then
		return true;
	end
	return false;
end

function JDI:Warn(msg, sound)
	RaidNotice_AddMessage(RaidBossEmoteFrame, msg, ChatTypeInfo["RAID_WARNING"]);
	PlaySoundFile(sound);
end

SlashCmdList['JDI_GUI'] = JDI.GUI.MainFrame.Toggle;

_G['SLASH_JDI_GUI1'] = '/joustdoit';
_G['SLASH_JDI_GUI2'] = '/joust';